--[[
Copyright (C) GtX (Andy), 2019

Author: GtX | Andy
Date: 04.01.2019
Revision: FS22-02

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy
Copying or removing any part of this code for external use without written permission from GtX | Andy is prohibited.

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
Das Kopieren oder Entfernen irgendeines Teils dieses Codes zur externen Verwendung ohne schriftliche Genehmigung von GtX | Andy ist verboten.
]]


TransferOwnershipManager = {}
TransferOwnershipManager.DEBUG_ENABLED = false

local TransferOwnershipManager_mt = Class(TransferOwnershipManager)

local modDirectory = g_currentModDirectory
local modName = g_currentModName

local versionString = "0.0.0.0"
local buildId = 1

local validationFail

function TransferOwnershipManager.new()
    local self = setmetatable({}, TransferOwnershipManager_mt)

    self.isServer = g_server ~= nil
    self.isClient = g_client ~= nil

    self.versionString = versionString
    self.buildId = buildId

    self.modName = modName

    return self
end

function TransferOwnershipManager:load()
    self:loadGui(false)

    g_messageCenter:subscribe("TRANSFER_OWNERSHIP_UPDATE_OBJECTS", self.onUpdateObjects, self)

    return true
end

function TransferOwnershipManager:loadGui(reload)
    if self.transferOwnershipScreen ~= nil and not reload then
        return false
    end

    g_gui:loadProfiles(modDirectory .. "gui/guiProfiles.xml")

    local transferOwnershipScreen = TransferOwnershipScreen.new()
    g_gui:loadGui(modDirectory .. "gui/TransferOwnershipScreen.xml", "TransferOwnershipScreen", transferOwnershipScreen)

    self.transferOwnershipScreen = transferOwnershipScreen
    g_transferOwnershipScreen = transferOwnershipScreen

    return true
end

function TransferOwnershipManager:delete()
    g_messageCenter:unsubscribeAll(self)
end

function TransferOwnershipManager:onUpdateObjects(transferPoint)
    local mission = g_currentMission

    if mission ~= nil and mission.shopController ~= nil then
        local farmId = mission:getFarmId()

        mission.shopController:setOwnedFarmItems(mission.ownedItems, farmId)
        mission.shopController:setLeasedFarmItems(mission.leasedVehicles, farmId)
    end
end

local function validateMod()
    local mod = g_modManager:getModByName(modName)

    if mod == nil or g_iconGenerator ~= nil then
        return true
    end

    versionString = mod.version or versionString

    if mod.modName == "FS22_TransferOwnership" or mod.modName == "FS22_TransferOwnership_update" then
        if mod.author ~= nil and #mod.author == 3 then
            return true
        end
    end

    validationFail = {
        startUpdateTime = 2000,

        update = function(self, dt)
            self.startUpdateTime = self.startUpdateTime - dt

            if self.startUpdateTime < 0 then
                local text = string.format(g_i18n:getText("transferOwnership_loadError", mod.modName), mod.modName, mod.author or "Unknown")

                if g_dedicatedServerInfo == nil then
                    if not g_gui:getIsGuiVisible() then
                        g_gui:showYesNoDialog({
                            title = string.format("%s - Version %s", mod.title, versionString),
                            text = text,
                            dialogType = DialogElement.TYPE_LOADING,
                            callback = self.openModHubLink,
                            yesText = g_i18n:getText("button_ok"),
                            noText = g_i18n:getText("button_modHubDownload")
                        })
                    end
                else
                    print("\n" .. text .. "\n    - https://farming-simulator.com/mods.php?lang=en&country=be&title=fs2022&filter=org&org_id=129652&page=0" .. "\n")
                    self.openModHubLink(true)
                end
            end
        end,

        openModHubLink = function(ignore)
            if ignore == false then
                local language = g_languageShort
                local link = "mods.php?lang=en&country=be&title=fs2022&filter=org&org_id=129652&page=0"
                if language == "de" or language == "fr" then
                    link = "mods.php?lang=" .. language .. "&country=be&title=fs2022&filter=org&org_id=129652&page=0"
                end

                openWebFile(link, "")
            end

            removeModEventListener(validationFail)
            validationFail = nil
        end
    }

    addModEventListener(validationFail)

    return false
end

local function init()
    if validateMod() then
        source(modDirectory .. "scripts/events/TransferOwnershipEvent.lua")
        source(modDirectory .. "scripts/events/TransferOwnershipUpdateObjectsEvent.lua")

        source(modDirectory .. "scripts/TransferOwnershipPoint.lua")
        source(modDirectory .. "scripts/gui/TransferOwnershipScreen.lua")

        local transferOwnershipManager = TransferOwnershipManager.new()

        FSBaseMission.loadMapFinished = Utils.prependedFunction(FSBaseMission.loadMapFinished, function(mission)
            if transferOwnershipManager ~= nil and transferOwnershipManager:load() then
                mission:addNonUpdateable(transferOwnershipManager)
            end
        end)

        g_transferOwnershipManager = transferOwnershipManager
    end
end

init()
