--[[
Copyright (C) GtX (Andy), 2021

Author: GtX | Andy
Date: 14.04.2021
Revision: FS22-01

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy
Copying or removing any part of this code for external use without written permission from GtX | Andy is prohibited.

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
Das Kopieren oder Entfernen irgendeines Teils dieses Codes zur externen Verwendung ohne schriftliche Genehmigung von GtX | Andy ist verboten.
]]


TransferOwnershipEvent = {}
TransferOwnershipEvent.SEND_NUM_BITS = 8

local TransferOwnershipEvent_mt = Class(TransferOwnershipEvent, Event)
InitEventClass(TransferOwnershipEvent, "TransferOwnershipEvent")

function TransferOwnershipEvent.emptyNew()
    local self = Event.new(TransferOwnershipEvent_mt)

    return self
end

function TransferOwnershipEvent.new(objects, destinationFarmId, owningPlaceable)
    local self = TransferOwnershipEvent.emptyNew()

    self.objects = objects or {}
    self.destinationFarmId = destinationFarmId
    self.owningPlaceable = owningPlaceable

    return self
end

function TransferOwnershipEvent:readStream(streamId, connection)
    local numObjects = streamReadUIntN(streamId, TransferOwnershipEvent.SEND_NUM_BITS)

    self.objects = {}

    if numObjects > 0 then
        for i = 1, numObjects do
            local object = NetworkUtil.readNodeObject(streamId)

            if object ~= nil then
                table.insert(self.objects, object)
            end
        end
    end

    self.destinationFarmId = streamReadUIntN(streamId, FarmManager.FARM_ID_SEND_NUM_BITS)
    self.owningPlaceable = NetworkUtil.readNodeObject(streamId)

    self:run(connection)
end

function TransferOwnershipEvent:writeStream(streamId, connection)
    local numObjects = math.min(#self.objects, 2 ^ TransferOwnershipEvent.SEND_NUM_BITS - 1)

    streamWriteUIntN(streamId, numObjects, TransferOwnershipEvent.SEND_NUM_BITS)

    if numObjects > 0 then
        for i = 1, numObjects do
            NetworkUtil.writeNodeObject(streamId, self.objects[i])
        end
    end

    streamWriteUIntN(streamId, self.destinationFarmId, FarmManager.FARM_ID_SEND_NUM_BITS)
    NetworkUtil.writeNodeObject(streamId, self.owningPlaceable)
end

function TransferOwnershipEvent:run(connection)
    if not connection:getIsServer() then
        local senderFarmId = g_currentMission:getFarmId(connection) or FarmManager.SPECTATOR_FARM_ID

        if g_currentMission:getHasPlayerPermission("transferMoney", connection, senderFarmId) and self.objects ~= nil then
            local destinationFarmId = self.destinationFarmId
            local transferPoint, numObjects = nil, 0

            if destinationFarmId ~= nil and g_farmManager:getFarmById(destinationFarmId) ~= nil then
                for _, object in ipairs (self.objects) do
                    if object.findRootVehicle ~= nil then
                        local rootVehicle = object:findRootVehicle() or object

                        if rootVehicle.getAttachedImplements ~= nil then
                            local implements = rootVehicle:getAttachedImplements()
                            local numImplements = implements ~= nil and #implements or 0

                            if numImplements > 0 then
                                for i = numImplements, 1, -1 do
                                    rootVehicle:detachImplement(1)
                                end
                            end
                        end
                    end

                    object:setOwnerFarmId(destinationFarmId)

                    numObjects = numObjects + 1
                end
            end

            g_server:broadcastEvent(TransferOwnershipUpdateObjectsEvent.new(self.owningPlaceable))

            if self.owningPlaceable ~= nil and self.owningPlaceable.getTransferOwnershipPoint ~= nil then
                transferPoint = self.owningPlaceable:getTransferOwnershipPoint()
            end

            g_messageCenter:publish("TRANSFER_OWNERSHIP_UPDATE_OBJECTS", transferPoint)

            if TransferOwnershipManager.DEBUG_ENABLED then
                printf("  Debug: [TransferOwnership] Transferred %d objects from farmId '%d' to farmId '%d'!", numObjects, senderFarmId, destinationFarmId)
            end
        end
    else
        Logging.error("TransferOwnershipEvent is a client to server only event")
    end
end
