--[[
Copyright (C) GtX (Andy), 2021

Author: GtX | Andy
Date: 14.04.2021
Revision: FS22-01

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy
Copying or removing any part of this code for external use without written permission from GtX | Andy is prohibited.

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
Das Kopieren oder Entfernen irgendeines Teils dieses Codes zur externen Verwendung ohne schriftliche Genehmigung von GtX | Andy ist verboten.
]]


TransferOwnershipScreen = {}

TransferOwnershipScreen.MOD_NAME = g_currentModName

TransferOwnershipScreen.CONTROLS = {
    "headerText",
    "list",
    "objectsBox",
    "objectImage",
    "objectName",
    "objectValueText",
    "numObjectsRow",
    "numObjectsTitle",
    "numObjectsText",
    "objectsFillLevelRow",
    "objectsFillLevelText",
    "screenInfo",
    "buttonBox",
    "transferButton"
}

local TransferOwnershipScreen_mt = Class(TransferOwnershipScreen, ScreenElement)

local function sortObjectSectionItems(a, b)
    return a.name < b.name
end

function TransferOwnershipScreen.new(target)
    local self = TransferOwnershipScreen:superClass().new(target, TransferOwnershipScreen_mt)

    self:registerControls(TransferOwnershipScreen.CONTROLS)

    self.objectSectionVehicles = {
        title = g_i18n:getText("ui_vehicles"),
        isVehicle = true,
        items = {}
    }

    self.objectSectionPallets = {
        title = g_i18n:getText("category_pallets"),
        isPallets = true,
        items = {}
    }

    self.objectSectionBales = {
        title = g_i18n:getText("category_bales"),
        isBales = true,
        items = {}
    }

    self.objectSections = {
        self.objectSectionVehicles,
        self.objectSectionPallets,
        self.objectSectionBales
    }

    self.defaultHeaderText = string.format(g_i18n:getText("button_mp_transferMoney_dialogTitle"), g_i18n:getText("ui_farm"))

    return self
end

function TransferOwnershipScreen:onOpen()
    TransferOwnershipScreen:superClass().onOpen(self)

    self:setHeaderText()

    g_messageCenter:subscribe("TRANSFER_OWNERSHIP_UPDATE_OBJECTS", self.updateObjects, self)
end

function TransferOwnershipScreen:onClose()
    TransferOwnershipScreen:superClass().onClose(self)

    self.transferPoint = nil
    self:setObjects()

    g_messageCenter:unsubscribeAll(self)
end

function TransferOwnershipScreen:update(dt)
    TransferOwnershipScreen:superClass().update(self, dt)

    if self.selectedObject ~= nil then
        if self.selectedObject.vehicle ~= nil then
            if self.selectedObject.vehicle.isDeleted then
                self:setObjects()
            end
        elseif self.selectedObject.objects ~= nil then
            local isBales = self.selectedObject.isBales

            for _, object in ipairs (self.selectedObject.objects) do
                if isBales then
                    if not entityExists(object.nodeId or 0) then
                        self:setObjects()

                        break
                    end
                elseif object.isDeleted then
                    self:setObjects()

                    break
                end
            end
        end
    end
end

function TransferOwnershipScreen:getNumberOfSections()
    return #self.objectSections
end

function TransferOwnershipScreen:getNumberOfItemsInSection(list, section)
    return #self.objectSections[section].items
end

function TransferOwnershipScreen:getTitleForSectionHeader(list, section)
    return self.objectSections[section].title
end

function TransferOwnershipScreen:onListSelectionChanged(list, section, index)
    self:setObject(self.objectSections[section].items[index])
end

function TransferOwnershipScreen:populateCellForItemInSection(list, section, index, cell)
    local object = self.objectSections[section].items[index]

    cell:getAttribute("icon"):setImageFilename(object.imageFilename or "dataS/menu/blank.dds")
    cell:getAttribute("name"):setText(object.name or "Unknown")

    if object.typeName ~= nil then
        local numObjectsValue = ""

        cell:getAttribute("typeName"):setText(object.typeName)

        if object.vehicle == nil and object.objects ~= nil then
            numObjectsValue = #object.objects

            if numObjectsValue < 2 then
                numObjectsValue = ""
            end
        end

        cell:getAttribute("numObjects"):setText(numObjectsValue)
    else
        cell:getAttribute("typeName"):setText("")
        cell:getAttribute("numObjects"):setText("")
    end
end

function TransferOwnershipScreen:setTransferPoint(transferPoint)
    if transferPoint == nil or transferPoint.determineCurrentObjects == nil then
        return false
    end

    self.transferPoint = transferPoint
    self:setObjects()

    return true
end

function TransferOwnershipScreen:setHeaderText(text)
    if self.headerText ~= nil then
        if text == nil then
            text = self.defaultHeaderText or ""
        end

        self.headerText:setText(text)
    end
end

function TransferOwnershipScreen:setObjects()
    local maxObjectsPerGroup = 2 ^ TransferOwnershipEvent.SEND_NUM_BITS - 1
    local vehicleItems, palletItems, baleItems = {}, {}, {}
    local vehicles, bales, hasItems = nil, nil, false

    if self.transferPoint ~= nil and self.transferPoint.determineCurrentObjects ~= nil then
        local farmId = g_currentMission:getFarmId()

        vehicles, bales = self.transferPoint:determineCurrentObjects()

        if vehicles ~= nil then
            local palletGroups = {}

            local bigBagText = g_i18n:getText("shopItem_bigBag")
            local palletText = g_i18n:getText("infohud_pallet")

            for _, vehicle in ipairs (vehicles) do
                if vehicle.ownerFarmId == farmId then
                    if vehicle.isPallet and vehicle.spec_pallet ~= nil then
                        local fillUnit = vehicle:getFillUnitByIndex(vehicle.spec_pallet.fillUnitIndex)
                        local fillTypeIndex = vehicle:getFillUnitFillType(vehicle.spec_pallet.fillUnitIndex)
                        local fillType = g_fillTypeManager:getFillTypeByIndex(fillTypeIndex)

                        if fillUnit ~= nil and fillType ~= nil then
                            local isBigBag = SpecializationUtil.hasSpecialization(BigBag, vehicle.specializations)
                            local typeKey, nameI18N = isBigBag and "BIGBAG" or "PALLET", nil

                            if vehicle.spec_treeSaplingPallet ~= nil then
                                local treeTypeDesc = vehicle.spec_treeSaplingPallet.treeTypeDesc

                                if treeTypeDesc ~= nil then
                                    typeKey = treeTypeDesc.name
                                    nameI18N = treeTypeDesc.nameI18N
                                end
                            end

                            local key = string.format("%d_%s", fillTypeIndex, typeKey)

                            if palletGroups[key] == nil then
                                local name = fillType.title

                                if nameI18N ~= nil then
                                    name = string.format("%s (%s)", name, g_i18n:getText(nameI18N, vehicle.customEnvironment))
                                end

                                palletGroups[key] = {
                                    imageFilename = fillType.hudOverlayFilename,
                                    fillTypeIndex = fillTypeIndex,
                                    name = name,
                                    unitText = fillUnit.unitText,
                                    typeName = isBigBag and bigBagText or palletText,
                                    objects = {},
                                    isBigBags = isBigBag,
                                    isPallets = true
                                }

                                table.insert(palletItems, palletGroups[key])
                            end

                            if #palletGroups[key].objects < maxObjectsPerGroup then
                                table.insert(palletGroups[key].objects, vehicle)
                            end
                        end
                    else
                        local vehicleObject = {
                            imageFilename = vehicle:getImageFilename(),
                            name = vehicle:getFullName(),
                            typeName = "",
                            vehicle = vehicle,
                            isVehicle = true
                        }

                        local brand = g_brandManager:getBrandByIndex(vehicle:getBrand())

                        if brand ~= nil then
                            vehicleObject.typeName = brand.title
                        end

                        table.insert(vehicleItems, vehicleObject)
                    end
                end
            end

            if #vehicleItems > 0 then
                hasItems = true
                table.sort(vehicleItems, sortObjectSectionItems)
            end

            if #palletItems > 0 then
                hasItems = true
                table.sort(palletItems, sortObjectSectionItems)
            end
        end

        if bales ~= nil then
            local baleGroups = {}

            local roundText = g_i18n:getText("fillType_roundBale")
            local squareText = g_i18n:getText("fillType_squareBale")
            local fermentingText = g_i18n:getText("info_fermenting")

            -- Make English plural, not worth translating just want it for me :-)
            if g_languageShort == "en" then
                roundText = roundText .. "s"
                squareText = squareText .. "s"
            end

            for _, bale in ipairs (bales) do
                if bale.ownerFarmId == farmId then
                    local fillTypeIndex = bale:getFillType()
                    local fillType = g_fillTypeManager:getFillTypeByIndex(fillTypeIndex)

                    if fillType ~= nil then
                        local key = string.format("%d_%s_%s", fillTypeIndex, bale.isRoundbale, bale.isFermenting)

                        if baleGroups[key] == nil then
                            local name = fillType.title

                            if bale.isFermenting then
                                name = string.format("%s (%s)", name, fermentingText)
                            end

                            baleGroups[key] = {
                                imageFilename = fillType.hudOverlayFilename,
                                fillTypeIndex = fillTypeIndex,
                                name = name,
                                typeName = bale.isRoundbale and roundText or squareText,
                                objects = {},
                                isBales = true
                            }

                            table.insert(baleItems, baleGroups[key])
                        end

                        if #baleGroups[key].objects < maxObjectsPerGroup then
                            table.insert(baleGroups[key].objects, bale)
                        end
                    end
                end
            end

            if #baleItems > 0 then
                hasItems = true
                table.sort(baleItems, sortObjectSectionItems)
            end
        end
    end

    self.objectSectionVehicles.items = vehicleItems
    self.objectSectionPallets.items = palletItems
    self.objectSectionBales.items = baleItems

    self.vehicles = vehicles
    self.bales = bales

    self.list:reloadData()

    if not hasItems then
        self:setObject(nil)
    end
end

function TransferOwnershipScreen:updateObjects(transferPoint)
    if transferPoint ~= nil and transferPoint == self.transferPoint then
        self:setObjects()
    end
end

function TransferOwnershipScreen:setObject(selectedObject)
    self.selectedObject = selectedObject

    if selectedObject ~= nil then
        local name = selectedObject.name or "Unknown"
        local value = "-"

        local vehicle = selectedObject.vehicle
        local objects = selectedObject.objects
        local numObjectsVisible = false
        local fillLevel = 0

        if vehicle ~= nil then
            if vehicle.propertyState == Vehicle.PROPERTY_STATE_OWNED then
                local sellPrice = math.floor(vehicle:getSellPrice() * EconomyManager.DIRECT_SELL_MULTIPLIER)
                value = g_i18n:formatMoney(math.min(sellPrice, vehicle:getPrice()))
            elseif vehicle.propertyState == Vehicle.PROPERTY_STATE_LEASED then
                value = g_i18n:getText("ui_lease") -- Future use
            end
        elseif objects ~= nil and #objects > 0 then
            local fillTypeIndex = selectedObject.fillTypeIndex

            if fillTypeIndex ~= nil then
                numObjectsVisible = true

                local highestSellPrice = 0

                for _, unloadingStation in pairs(g_currentMission.storageSystem:getUnloadingStations()) do
                    if unloadingStation.owningPlaceable ~= nil and unloadingStation.isSellingPoint and unloadingStation.acceptedFillTypes[fillTypeIndex] then
                        local price = unloadingStation:getEffectiveFillTypePrice(fillTypeIndex)

                        if price > 0 then
                            highestSellPrice = math.max(highestSellPrice, price)
                        end
                    end
                end

                if selectedObject.isPallets then
                    for _, object in ipairs (selectedObject.objects) do
                        fillLevel = fillLevel + (object:getFillUnitFillLevel(object.spec_pallet.fillUnitIndex) or 0)
                    end
                elseif selectedObject.isBales then
                    for _, object in ipairs (selectedObject.objects) do
                        fillLevel = fillLevel + (object:getFillLevel() or 0)
                    end
                end

                if highestSellPrice > 0 and fillLevel > 0 then
                    value = g_i18n:formatMoney(fillLevel * highestSellPrice)
                end

                local numObjectsTitle = "-"

                if selectedObject.isPallets then
                    if selectedObject.isBigBags then
                        numObjectsTitle = g_i18n:getText("category_bigbags")
                    else
                        numObjectsTitle = g_i18n:getText("category_pallets")
                    end
                else
                    if selectedObject.typeName ~= nil then
                        numObjectsTitle = selectedObject.typeName
                    else
                        numObjectsTitle = g_i18n:getText("category_bales")
                    end
                end

                self.numObjectsTitle:setText(numObjectsTitle)
                self.numObjectsText:setText(#selectedObject.objects)

                self.objectsFillLevelText:setText(g_i18n:formatVolume(fillLevel, 0, selectedObject.unitText))
            end
        end

        self.objectImage:setImageFilename(selectedObject.imageFilename or "dataS/menu/blank.dds")
        self.objectName:setText(string.upper(name))
        self.objectValueText:setText(value)

        self.numObjectsRow:setVisible(numObjectsVisible)
        self.objectsFillLevelRow:setVisible(fillLevel > 0)
    end

    self.screenInfo:setVisible(selectedObject == nil)
    self.objectsBox:setVisible(selectedObject ~= nil)
end

function TransferOwnershipScreen:onClickBack(forceBack, usedMenuButton)
    self:changeScreen(nil)

    return true
end

function TransferOwnershipScreen:onClickTransfer()
    if not g_currentMission.missionDynamicInfo.isMultiplayer then
        g_gui:showInfoDialog({
            text = string.format("%s\n\n%s", g_i18n:getText("warning_actionNotAllowedNow"), g_i18n:getText("ui_joinMultiplayerGame")),
            dialogType = DialogElement.TYPE_INFO,
            okText = g_i18n:getText("button_back"),
            buttonAction = InputAction.MENU_BACK
        })

        self:playSample(GuiSoundPlayer.SOUND_SAMPLES.ERROR)

        return true
    end

    local senderFarmId = g_currentMission:getFarmId()

    if self.selectedObject ~= nil then
        local objectsToSend = {}

        if self.selectedObject.objects ~= nil then
            for _, object in ipairs (self.selectedObject.objects) do
                table.insert(objectsToSend, object)
            end
        elseif self.selectedObject.vehicle ~= nil then
            table.insert(objectsToSend, self.selectedObject.vehicle)
        end

        if #objectsToSend > 0 and senderFarmId ~= FarmManager.SPECTATOR_FARM_ID then
            local farmNames = {}
            local farms = {}

            for _, farm in ipairs (g_farmManager:getFarms()) do
                if farm.farmId ~= senderFarmId and farm.farmId ~= FarmManager.SPECTATOR_FARM_ID then
                    table.insert(farmNames, farm.name)
                    table.insert(farms, farm)
                end
            end

            if #farms > 0 then
                g_gui:showOptionDialog({
                    title = self.headerText:getText(),
                    text = g_i18n:getText("ui_transferOwnershipSelectFarm", TransferOwnershipScreen.MOD_NAME),
                    options = farmNames,
                    callback = function (index)
                        if index > 0 then
                            local farm = farms[index]

                            if farm ~= nil and g_farmManager:getFarmById(farm.farmId) ~= nil then
                                if self:onConfirmTransfer(senderFarmId, farm.farmId, objectsToSend) then
                                    self:playSample(GuiSoundPlayer.SOUND_SAMPLES.TRANSACTION)
                                end
                            end
                        end
                    end
                })
            else
                g_gui:showInfoDialog({
                    text = g_i18n:getText("ui_noFarmsCreated"),
                    dialogType = DialogElement.TYPE_INFO,
                    okText = g_i18n:getText("button_back"),
                    buttonAction = InputAction.MENU_BACK
                })

                self:playSample(GuiSoundPlayer.SOUND_SAMPLES.ERROR)
            end

            return false
        end
    end

    self:setObjects()

    return true
end

function TransferOwnershipScreen:onConfirmTransfer(senderFarmId, destinationFarmId, objects)
    if senderFarmId == nil or destinationFarmId == nil or g_farmManager:getFarmById(destinationFarmId) == nil or objects == nil then
        return false
    end

    local owningPlaceable = nil

    if self.transferPoint ~= nil then
        owningPlaceable = self.transferPoint.owningPlaceable
    end

    if g_server ~= nil then
        local numObjects = 0

        for _, object in ipairs (objects) do
            if object.findRootVehicle ~= nil then
                local rootVehicle = object:findRootVehicle() or object

                if rootVehicle.getAttachedImplements ~= nil then
                    local implements = rootVehicle:getAttachedImplements()
                    local numImplements = implements ~= nil and #implements or 0

                    if numImplements > 0 then
                        for i = numImplements, 1, -1 do
                            rootVehicle:detachImplement(1)
                        end
                    end
                end
            end

            object:setOwnerFarmId(destinationFarmId)

            numObjects = numObjects + 1
        end

        if TransferOwnershipManager.DEBUG_ENABLED then
            printf("  Debug: [TransferOwnership] Transferred %d objects from farmId '%d' to farmId '%d'!", numObjects, senderFarmId, destinationFarmId)
        end

        g_server:broadcastEvent(TransferOwnershipUpdateObjectsEvent.new(owningPlaceable))
        g_messageCenter:publish("TRANSFER_OWNERSHIP_UPDATE_OBJECTS", nil) -- Only trigger 'shopController' update so send nil for 'transferPoint'

        self:setObjects()
    else
        g_client:getServerConnection():sendEvent(TransferOwnershipEvent.new(objects, destinationFarmId, owningPlaceable))
    end

    return true
end
